from __future__ import annotations

from .dan.AngryTweetsClassification import *
from .dan.DalajClassification import *
from .dan.DanishPoliticalCommentsClassification import *
from .dan.DKHateClassification import *
from .dan.LccSentimentClassification import *
from .eng.AmazonPolarityClassification import *
from .eng.Banking77Classification import *
from .eng.EmotionClassification import *
from .eng.ImdbClassification import *
from .eng.NewsClassification import *
from .eng.ToxicConversationsClassification import *
from .eng.TweetSentimentExtractionClassification import *
from .multilingual.AmazonCounterfactualClassification import *
from .multilingual.AmazonReviewsClassification import *
from .multilingual.MasakhaNEWSClassification import *
from .multilingual.MassiveIntentClassification import *
from .multilingual.MassiveScenarioClassification import *
from .multilingual.MTOPDomainClassification import *
from .multilingual.MTOPIntentClassification import *
from .multilingual.NordicLangClassification import *
from .multilingual.ScalaClassification import *
from .nob.NoRecClassification import *
from .nob.NorwegianParliamentClassification import *
from .pol.PolishClassification import *
from .swe.SweRecClassification import *
from .zho.CMTEBClassification import *
