from __future__ import annotations

from .eng.SprintDuplicateQuestionsPC import *
from .eng.TwitterSemEval2015PC import *
from .eng.TwitterURLCorpusPC import *
from .multilingual.OpusparcusPC import *
from .multilingual.PawsX import *
from .pol.PolishPC import *
from .zho.CMTEBPairClassification import *
